package br.usp.ime.mac5855.knn;

/**
 * @author Victor Williams Stafusa da Silva
 */
public final class ComparadorEuclideano<E> implements ComparadorKnn<E> {

    private final Escalonador<E> escalonador;

    public ComparadorEuclideano(Escalonador<E> escalonador) {
        this.escalonador = escalonador;
    }

    @Override
    public double distancia(E elem1, E elem2) {
        double[] valores1 = escalonador.valores(elem1);
        double[] valores2 = escalonador.valores(elem2);

        if (valores1 == null || valores2 == null) {
            throw new IllegalArgumentException("A valorao dos elementos no  vlida. No pode ser null.");
        }

        if (valores1.length == 0 || valores2.length == 0) {
            throw new IllegalArgumentException("A valorao dos elementos no  vlida. O nmero de dimenses no pode ser zero.");
        }

        if (valores1.length != valores2.length) {
            throw new IllegalArgumentException("A valorao dos elementos no  vlida. O nmero de dimenses deve ser igual para todos os elementos.");
        }

        double distancia = 0.0;

        for (int i = 0; i < valores1.length; i++) {
            double valor = escalonador.peso(i, valores1[i] - valores2[i]);
            distancia += valor * valor;
        }
        return Math.sqrt(distancia);
    }
}