package br.usp.ime.mac5855.knn.validacaocruzada;

import br.usp.ime.mac5855.knn.KnnPonderado;
import br.usp.ime.mac5855.knn.KnnStrategy;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

/**
 * @author Victor Williams Stafusa da Silva
 */
public class ValidacaoCruzada<E> {

    private final ValidacaoCruzadaStrategy strategy;
    private final Map<E, Double> populacao;

    public ValidacaoCruzada(Map<E, Double> populacao, ValidacaoCruzadaStrategy strategy) {
        this.populacao = populacao;
        this.strategy = strategy;
    }

    private class TesteValidacaoCruzada<E> {
        private final Map<E, Double> teste;
        private final Map<E, Double> treinamento;

        public TesteValidacaoCruzada(Map<E, Double> teste, Map<E, Double> treinamento) {
            this.treinamento = treinamento;
            this.teste = teste;
        }

        public Map<E, Double> getTeste() {
            return teste;
        }

        public Map<E, Double> getTreinamento() {
            return treinamento;
        }
    }

    private TesteValidacaoCruzada<E> dividir(double proporcaoTeste) {
        if (proporcaoTeste < 0.0 || proporcaoTeste > 1.0) throw new AssertionError();
        int tamanhoPopulacao = populacao.size();
        int tamanhoTeste = (int) (tamanhoPopulacao * proporcaoTeste);
        int tamanhoTreinamento = tamanhoPopulacao - tamanhoTeste;

        Map<E, Double> treinamento = new HashMap<E, Double>();
        Map<E, Double> teste = new HashMap<E, Double>();
        Random r = strategy.random();

        for (Map.Entry<E, Double> entry : populacao.entrySet()) {
            if (r.nextInt(tamanhoPopulacao) < tamanhoTeste) {
                teste.put(entry.getKey(), entry.getValue());
                tamanhoTeste--;
            } else {
                treinamento.put(entry.getKey(), entry.getValue());
                tamanhoTreinamento--;
            }
            tamanhoPopulacao--;
        }
        return new TesteValidacaoCruzada<E>(teste, treinamento);
    }

    public double validar(int tentativas, double proporcaoTeste, KnnStrategy<E> knnStrategy) {
        if (tentativas <= 0) {
            throw new IllegalArgumentException("O nmero de tentativas deve ser pelo menos 1.");
        }
        if (proporcaoTeste < 0.0 || proporcaoTeste > 1.0) {
            throw new IllegalArgumentException("A proporo de teste deve estar entre 0.0 e 1.0");
        }

        double erro = 0.0;
        for (int i = 0; i < tentativas; i++) {
            TesteValidacaoCruzada<E> conjuntos = dividir(proporcaoTeste);
            KnnPonderado<E> knn = new KnnPonderado<E>(conjuntos.getTreinamento(), knnStrategy);
            for (Map.Entry<E, Double> entry : conjuntos.getTeste().entrySet()) {
                double precoEstimado = knn.estimarPreco(entry.getKey());
                erro += strategy.avaliarErro(precoEstimado, entry.getValue());
            }
        }
        return erro / tentativas;
    }
}